## 5/07/2018
## Analysis of the Loblolly Pine Survival Data
## This is R code fits the AFT GRF model in the paper
## To fit a IID frailty model, one just needs to use frailtyprior("iid", siteID)
rm(list=ls())
library(spBayesSurv)
library(survival)
library(coda)
library(fields)
library(maps)
library(ggplot2)
cleanup <- 
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(), 
        panel.background = element_rect(fill = 'white', colour = 'white'), 
        axis.line = element_line(colour = "white"), #legend.position="none",
        axis.ticks=element_blank(), axis.text.x=element_blank(),
        axis.text.y=element_blank())

############## Read table ##############
d=read.csv(file="tree.survival.data.JASA.csv");
d=d[order(d$SN),]
n = nrow(d); n; 
d$obs = 1:n;
## coordinates;
locations0 = d[, c("Long","Lat")];
locations = d[!duplicated(locations0), c("Long","Lat")];
plot(locations);
## Define Distance;
cor.dist = function(x1, x2) rdist.earth(x1,x2, miles=FALSE, R=6378.137)
#Dmm = cor.dist(locations, locations); quantile(Dmm)
## knots;
s0 = as.matrix(fields::cover.design(locations, nd=50, DIST=cor.dist)$design);
points(s0, col=2);
## group by unique locations
Dnm = rdist(locations0, locations);
d$siteID = apply(Dnm, 1, which.min);
order.ID = order(d$siteID);
d = d[order(d$siteID),];
unique_SN = unique(d$SN);
table_SN = sapply(unique_SN,function(x)sum(d$SN==x))
d$newSN = rep(1:length(unique_SN), table_SN)
d$treat2 = (d$Plot==2)+0;
d$treat3 = (d$Plot==3)+0;
d$PhyReg2 = (d$PhyReg==2)+0;
d$PhyReg3 = (d$PhyReg==3)+0;
d$C2 = (d$C==2)+0;
d$C3 = (d$C==3)+0;
d$C4 = (d$C==4)+0;
head(d); 

##-------------AFT-------------------##
# MCMC parameters
nburn=10000; nsave=2000; nskip=19; niter = nburn+nsave
mcmc=list(nburn=nburn, nsave=nsave, nskip=nskip, ndisplay=1000);
prior = list(maxL=15, a0=1, b0=1, nu=1);
ptm<-proc.time()
res2 = survregbayes(formula = Surv(Start, End, Delta)~DBH+TH+treat2+treat3+PhyReg2+PhyReg3+
                      C2+C3+C4+treat2:PhyReg2+treat3:PhyReg2+treat2:PhyReg3+treat3:PhyReg3+
                      frailtyprior("grf", siteID), data=d, dist="loglogistic", InitParamMCMC=TRUE,
                    survmodel="AFT", selection=F, prior=prior, mcmc=mcmc, 
                    Coordinates = locations, DIST=cor.dist, subject.num = newSN);
sfit2=summary(res2); sfit2
systime2=proc.time()-ptm; systime2;

save.image("ResultAFTgrf.RData")

############# traceplots ###############
par(mfrow = c(3,3))
traceplot(mcmc(res2$beta[1,]), main="");
traceplot(mcmc(res2$beta[2,]), main="");
traceplot(mcmc(res2$beta[3,]), main="");
traceplot(mcmc(res2$beta[4,]), main="");
traceplot(mcmc(res2$beta[5,]), main="");
traceplot(mcmc(res2$beta[6,]), main="");
traceplot(mcmc(res2$beta[7,]), main="");
traceplot(mcmc(res2$beta[8,]), main="");
traceplot(mcmc(res2$beta[9,]), main="");
traceplot(mcmc(res2$beta[10,]), main="");
traceplot(mcmc(res2$beta[11,]), main="");
traceplot(mcmc(res2$beta[12,]), main="");
traceplot(mcmc(res2$beta[13,]), main="");

########### Cox-Snell plots with 10 random draws ###########
set.seed(1)
pdf(file ="Loblolly-AFT-GRF-Cox-Snell.pdf", paper="special", width=8, height=8)
cox.snell.survregbayes(res2, ncurves = 10)
dev.off()

########### Frailty Map ##################
xlim=c(-124.68134+30, -67.00742-5);ylim=c(25.12993, 49.38323-10); width=7; height=7;
par(cex=1.2,mar=c(4.1,4.1,1,1),cex.lab=1.3,cex.axis=1.1);
par(mar=c(0.5,0,0,0))
map0 = map_data("state", boundary=TRUE, lty=1, xlim = xlim, ylim = ylim)
p <- ggplot() + coord_fixed() + xlab("") + ylab("")
p1=p + geom_polygon(data=map0, aes(x=long, y=lat, group=group),
                    colour="black", fill="light green", alpha=I(0.5));
newdata=data.frame(Longitude=-res2$Knots[,1], Latitude=res2$Knots[,2],
                   frailty=rowMeans(res2$v))
p2=p1+geom_point(data=newdata, 
                 aes(x=Longitude, y=Latitude, size=frailty), colour="red", 
                 fill="Pink",pch=21, alpha=I(0.8))+cleanup; p2;
pdf(file ="Loblolly-AFT-GRF-Map-frailty.pdf", paper="special", width=8, height=5)
par(mar=c(0.5,0,0,0), cex.lab=1.3,cex.axis=1.1, cex=1.2)
plot(p2)
dev.off()

########### Survival curves ##################
meanTH = mean(d$TH);
meanDBH = mean(d$DBH);
tgrid = seq(1e-3,20,0.05);
# treatment in region 1
xpred = data.frame(DBH = rep(meanDBH, 3), TH = rep(meanTH, 3), 
                   treat2 = c(0, 1, 0), treat3 = c(0, 0, 1), 
                   PhyReg2 = rep(0, 3), PhyReg3 = rep(0, 3), 
                   C2 = c(0, 0, 0), C3 = c(0, 0, 0), C4 = c(0, 0, 0)); 
estimates=plot(res2, xnewdata = xpred, tgrid = tgrid);
## plot
pdf(file ="Loblolly-AFT-GRF-treatment-region1.pdf", paper="special", width=8, height=5);
par(cex=1.2,mar=c(4.1,4.1,1,1),cex.lab=1.3,cex.axis=1.1)
plot(estimates$tgrid, estimates$Shat[,1], "l", lwd=3, ylim = c(0.75, 1),
     xlab = "Time (years)", ylab="Survival Probability")
lines(estimates$tgrid, estimates$Shat[,2], lty=2, lwd=3, col="red")
lines(estimates$tgrid, estimates$Shat[,3], lty=3, lwd=3, col="green")
legend(0.5,0.85, legend=c("control","light thinning","heavy thinning"), 
       lty=c(1,2,3), lwd=c(3,3,3), col=c("black","red","green"), bty="n", cex=1.2)
dev.off()
# treatment in region 2
xpred = data.frame(DBH = rep(meanDBH, 3), TH = rep(meanTH, 3), 
                   treat2 = c(0, 1, 0), treat3 = c(0, 0, 1), 
                   PhyReg2 = rep(1, 3), PhyReg3 = rep(0, 3), 
                   C2 = c(0, 0, 0), C3 = c(0, 0, 0), C4 = c(0, 0, 0)); 
estimates=plot(res2, xnewdata = xpred, tgrid = tgrid);
## plot
pdf(file ="Loblolly-AFT-GRF-treatment-region2.pdf", paper="special", width=8, height=5);
par(cex=1.2,mar=c(4.1,4.1,1,1),cex.lab=1.3,cex.axis=1.1)
plot(estimates$tgrid, estimates$Shat[,1], "l", lwd=3, ylim = c(0.75, 1),
     xlab = "Time (years)", ylab="Survival Probability")
lines(estimates$tgrid, estimates$Shat[,2], lty=2, lwd=3, col="red")
lines(estimates$tgrid, estimates$Shat[,3], lty=3, lwd=3, col="green")
legend(0.5,0.85, legend=c("control","light thinning","heavy thinning"), 
       lty=c(1,2,3), lwd=c(3,3,3), col=c("black","red","green"), bty="n", cex=1.2)
dev.off()
# treatment in region 3
xpred = data.frame(DBH = rep(meanDBH, 3), TH = rep(meanTH, 3), 
                   treat2 = c(0, 1, 0), treat3 = c(0, 0, 1), 
                   PhyReg2 = rep(0, 3), PhyReg3 = rep(1, 3), 
                   C2 = c(0, 0, 0), C3 = c(0, 0, 0), C4 = c(0, 0, 0)); 
estimates=plot(res2, xnewdata = xpred, tgrid = tgrid);
## plot
pdf(file ="Loblolly-AFT-GRF-treatment-region3.pdf", paper="special", width=8, height=5);
par(cex=1.2,mar=c(4.1,4.1,1,1),cex.lab=1.3,cex.axis=1.1)
plot(estimates$tgrid, estimates$Shat[,1], "l", lwd=3, ylim = c(0.75, 1),
     xlab = "Time (years)", ylab="Survival Probability")
lines(estimates$tgrid, estimates$Shat[,2], lty=2, lwd=3, col="red")
lines(estimates$tgrid, estimates$Shat[,3], lty=3, lwd=3, col="green")
legend(0.5,0.85, legend=c("control","light thinning","heavy thinning"), 
       lty=c(1,2,3), lwd=c(3,3,3), col=c("black","red","green"), bty="n", cex=1.2)
dev.off()
# Crown class
xpred = data.frame(DBH = rep(meanDBH, 4), TH = rep(meanTH, 4), 
                   treat2 = rep(0, 4), treat3 = rep(0, 4), 
                   PhyReg2 = rep(0, 4), PhyReg3 = rep(0, 4), 
                   C2 = c(0, 1, 0, 0), C3 = c(0, 0, 1, 0), C4 = c(0, 0, 0, 1)); 
estimates=plot(res2, xnewdata = xpred, tgrid = tgrid);
## plot
pdf(file ="Loblolly-AFT-GRF-crown.pdf", paper="special", width=7, height=7);
par(cex=1.2,mar=c(4.1,4.1,1,1),cex.lab=1.3,cex.axis=1.1)
plot(estimates$tgrid, estimates$Shat[,1], "l", lwd=3, ylim = c(0, 1),
     xlab = "Time (years)", ylab="Survival Probability")
lines(estimates$tgrid, estimates$Shat[,2], lty=2, lwd=3, col="red")
lines(estimates$tgrid, estimates$Shat[,3], lty=3, lwd=3, col="green")
lines(estimates$tgrid, estimates$Shat[,4], lty=4, lwd=3, col="purple")
legend(0.5,0.25, legend=c("dominant","codominant","intermediate","suppressed"), 
       lty=c(1,2,3,4), lwd=c(3,3,3,3), col=c("black","red","green","purple"), bty="n", cex=1.2)
dev.off()
